/**
 * @file flock.hpp
 * @brief Function declarations for flocks.
 * @note Please see David Baraff's rigid body paper for more details
 *
 * @author Kristin Siu (kasiu)
 */

#ifndef _462_GAME_FLOCK_HPP_
#define _462_GAME_FLOCK_HPP_

#include "application/opengl.hpp"
#include "math/vector.hpp"
#include "math/quaternion.hpp"
#include "math/matrix.hpp"
#include "math/camera.hpp"
#include "game/integrator.hpp"
#include <string>
#include <vector>

namespace _462 {

/**
 * A particle representing a flocking boid.
 */
struct Boid
{ 
    // particle members, part of integrator state
    
    // the position
    Vector3 position;
    // the velocity
    Vector3 velocity;

    // other info, not part of integrator state

    // force accumulator
    Vector3 force_accumulator;
    // mass
    real_t mass;

    // TODO anything else you need (e.g., radius, field of view)
};

class Flock : public IntegrableSystem
{
public:

    virtual ~Flock() { }

    // init code
    /// Initialize the flock. Returns true on success.
    bool initialize( size_t num_boids, const Vector3& goal );
    /// The the initial data for a specific boid.
    /// TODO you'll probably need to add some additional parameters, like radius and fov.
    void set_boid( size_t index, const Vector3& position, const Vector3& velocity, real_t mass );

    /// Render the boids.
    void render() const;

    /// Set the goal (for goal seeking behavior) to the given value.
    void set_goal( const Vector3& goal );

    // integrable system interface

    virtual size_t size() const;
    virtual void get_state( real_t* arr, real_t* time ) const;
    virtual void set_state( const real_t* arr, const real_t time );
    virtual void eval_deriv( real_t* deriv_result );

private:

    typedef std::vector< Boid > BoidList;

    // the list of all boids
    BoidList boids;

    // the current time state
    real_t time;

    // the goal
    Vector3 goal;
};

} /* _462 */

#endif /* _462_GAME_FLOCK_HPP_ */
